@@ -3,6 +3,7 @@ from __future__ import annotations
 import ast
 import builtins
 import sys
+import typing
 from ast import (
     AST,
     Add,
@@ -136,6 +137,7 @@ class TransformMemo:
     should_instrument: bool = field(init=False, default=True)
     variable_annotations: dict[str, expr] = field(init=False, default_factory=dict)
     configuration_overrides: dict[str, Any] = field(init=False, default_factory=dict)
+    code_inject_index: int = field(init=False, default=0)
 
     def __post_init__(self) -> None:
         elements: list[str] = []
@@ -151,6 +153,23 @@ class TransformMemo:
 
         self.joined_path = Constant(".".join(elements))
 
+        # Figure out where to insert instrumentation code
+        if self.node:
+            for index, child in enumerate(self.node.body):
+                if isinstance(child, ImportFrom) and child.module == "__future__":
+                    # (module only) __future__ imports must come first
+                    continue
+                elif isinstance(child, Expr):
+                    if isinstance(child.value, Constant) and isinstance(
+                        child.value.value, str
+                    ):
+                        continue  # docstring
+                    elif sys.version_info < (3, 8) and isinstance(child.value, Str):
+                        continue  # docstring
+
+                self.code_inject_index = index
+                break
+
     def get_unused_name(self, name: str) -> str:
         memo: TransformMemo | None = self
         while memo is not None:
@@ -211,20 +230,12 @@ class TransformMemo:
             return
 
         # Insert imports after any "from __future__ ..." imports and any docstring
-        for i, child in enumerate(node.body):
-            if isinstance(child, ImportFrom) and child.module == "__future__":
-                continue
-            elif isinstance(child, Expr) and isinstance(child.value, Str):
-                continue  # module docstring
-
-            for modulename, names in self.load_names.items():
-                aliases = [
-                    alias(orig_name, new_name.id if orig_name != new_name.id else None)
-                    for orig_name, new_name in sorted(names.items())
-                ]
-                node.body.insert(i, ImportFrom(modulename, aliases, 0))
-
-            break
+        for modulename, names in self.load_names.items():
+            aliases = [
+                alias(orig_name, new_name.id if orig_name != new_name.id else None)
+                for orig_name, new_name in sorted(names.items())
+            ]
+            node.body.insert(self.code_inject_index, ImportFrom(modulename, aliases, 0))
 
     def name_matches(self, expression: expr | Expr | None, *names: str) -> bool:
         if expression is None:
@@ -352,6 +363,12 @@ class AnnotationTransformer(NodeTransformer):
 
         return new_node
 
+    def generic_visit(self, node: AST) -> AST:
+        if isinstance(node, expr) and self._memo.name_matches(node, *literal_names):
+            return node
+
+        return super().generic_visit(node)
+
     def visit_BinOp(self, node: BinOp) -> Any:
         self.generic_visit(node)
 
@@ -384,7 +401,7 @@ class AnnotationTransformer(NodeTransformer):
 
         # The subscript of typing(_extensions).Literal can be any arbitrary string, so
         # don't try to evaluate it as code
-        if not self._memo.name_matches(node.value, *literal_names) and node.slice:
+        if node.slice:
             if isinstance(node.slice, Index):
                 # Python 3.7 and 3.8
                 slice_value = node.slice.value  # type: ignore[attr-defined]
@@ -395,14 +412,23 @@ class AnnotationTransformer(NodeTransformer):
                 if self._memo.name_matches(node.value, *annotated_names):
                     # Only treat the first argument to typing.Annotated as a potential
                     # forward reference
-                    items = [self.visit(slice_value.elts[0])] + slice_value.elts[1:]
+                    items = cast(
+                        typing.List[expr],
+                        [self.generic_visit(slice_value.elts[0])]
+                        + slice_value.elts[1:],
+                    )
                 else:
-                    items = [self.visit(item) for item in slice_value.elts]
+                    items = cast(
+                        typing.List[expr],
+                        [self.generic_visit(item) for item in slice_value.elts],
+                    )
 
-                # If this is a Union and any of the items is None, erase the entire
+                # If this is a Union and any of the items is Any, erase the entire
                 # annotation
                 if self._memo.name_matches(node.value, "typing.Union") and any(
-                    item is None for item in items
+                    isinstance(item, expr)
+                    and self._memo.name_matches(item, *anytype_names)
+                    for item in items
                 ):
                     return None
 
@@ -468,10 +494,14 @@ class AnnotationTransformer(NodeTransformer):
 
 
 class TypeguardTransformer(NodeTransformer):
-    def __init__(self, target_path: Sequence[str] | None = None) -> None:
+    def __init__(
+        self, target_path: Sequence[str] | None = None, target_lineno: int | None = None
+    ) -> None:
         self._target_path = tuple(target_path) if target_path else None
         self._memo = self._module_memo = TransformMemo(None, None, ())
         self.names_used_in_annotations: set[str] = set()
+        self.target_node: FunctionDef | AsyncFunctionDef | None = None
+        self.target_lineno = target_lineno
 
     @contextmanager
     def _use_memo(
@@ -644,6 +674,12 @@ class TypeguardTransformer(NodeTransformer):
         with self._use_memo(node):
             arg_annotations: dict[str, Any] = {}
             if self._target_path is None or self._memo.path == self._target_path:
+                # Find line number we're supposed to match against
+                if node.decorator_list:
+                    first_lineno = node.decorator_list[0].lineno
+                else:
+                    first_lineno = node.lineno
+
                 for decorator in node.decorator_list.copy():
                     if self._memo.name_matches(decorator, "typing.overload"):
                         # Remove overloads entirely
@@ -658,6 +694,14 @@ class TypeguardTransformer(NodeTransformer):
                                 kw.arg: kw.value for kw in decorator.keywords if kw.arg
                             }
 
+                if self.target_lineno == first_lineno:
+                    assert self.target_node is None
+                    self.target_node = node
+                    if node.decorator_list and sys.version_info >= (3, 8):
+                        self.target_lineno = node.decorator_list[0].lineno
+                    else:
+                        self.target_lineno = node.lineno
+
                 all_args = node.args.args + node.args.kwonlyargs
                 if sys.version_info >= (3, 8):
                     all_args.extend(node.args.posonlyargs)
@@ -747,7 +791,9 @@ class TypeguardTransformer(NodeTransformer):
                     annotations_dict,
                     self._memo.get_memo_name(),
                 ]
-                node.body.insert(0, Expr(Call(func_name, args, [])))
+                node.body.insert(
+                    self._memo.code_inject_index, Expr(Call(func_name, args, []))
+                )
 
             # Add a checked "return None" to the end if there's no explicit return
             # Skip if the return annotation is None or Any
@@ -849,7 +895,7 @@ class TypeguardTransformer(NodeTransformer):
                     [keyword(key, value) for key, value in memo_kwargs.items()],
                 )
                 node.body.insert(
-                    0,
+                    self._memo.code_inject_index,
                     Assign([memo_store_name], memo_expr),
                 )
 
@@ -902,7 +948,6 @@ class TypeguardTransformer(NodeTransformer):
         self._memo.has_yield_expressions = True
         self.generic_visit(node)
 
-        self.generic_visit(node)
         if (
             self._memo.yield_annotation
             and self._memo.should_instrument
